package ch.unige.aou.controller.admin;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.security.UserPermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.settings.DownloadToken;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.service.DownloadTokenService;

@UserPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ACCESS_DOWNLOAD_TOKEN)
public class DownloadTokenController {

  private final DownloadTokenService downloadTokenService;

  public DownloadTokenController(DownloadTokenService downloadTokenService) {
    this.downloadTokenService = downloadTokenService;
  }

  @GetMapping("/{archiveId}")
  @PreAuthorize("@metadataPermissionService.isAllowed(#archiveId, 'DOWNLOAD_ARCHIVE') || @aclPermissionService.isAllowed(#archiveId)")
  public ResponseEntity<DownloadToken> getToken(@PathVariable String archiveId) {
    return new ResponseEntity<>(this.downloadTokenService.getHeadersWithToken(archiveId), HttpStatus.OK);
  }

}
