package ch.unige.aou.controller.admin;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;

import java.io.IOException;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;
import org.springframework.http.HttpEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.Tool;
import ch.unige.solidify.security.AdminPermissions;
import ch.unige.solidify.security.UserPermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.controller.AouResourceWithLogoController;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.security.CurrentUserPermissions;

@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_PEOPLE)
public class PersonController extends AouResourceWithLogoController<Person> {

  @Override
  @AdminPermissions
  public HttpEntity<Collection<Person>> advancedSearch(@ModelAttribute Person person,
          @RequestParam("search") String search,
          @RequestParam(value = "match", required = false) String matchtype, Pageable pageable) {
    return super.advancedSearch(person, search, matchtype, pageable);
  }

  @Override
  @AdminPermissions
  public HttpEntity<Collection<Person>> list(@ModelAttribute Person search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  @UserPermissions
  public HttpEntity<Person> get(@PathVariable String id) {
    return super.get(id);
  }

  @Override
  @CurrentUserPermissions
  public HttpEntity<Person> update(@PathVariable String id, @RequestBody Person person) {
    return super.update(id, person);
  }

  @Override
  protected Collection<Person> setCollectionLinksForMethod(Page<Person> listItem, Pageable pageable, WebMvcLinkBuilder linkBuilder) {
    final Collection<Person> collection = new Collection<>(listItem, pageable);
    collection.add(linkBuilder.withSelfRel());
    collection.add(Tool.parentLink((linkTo(this.getClass())).toUriComponentsBuilder()).withRel(ActionName.MODULE));
    this.addSortLinks(linkBuilder, collection);
    this.addPageLinks(linkBuilder, collection, pageable);
    this.addOthersLinks(collection);
    return collection;
  }

  @PostMapping(SolidifyConstants.URL_ID_PLUS_SEP + AouActionName.UPLOAD_AVATAR)
  @CurrentUserPermissions
  public HttpEntity<Person> uploadAvatar(@PathVariable String id, @RequestParam("file") MultipartFile file) throws IOException {
    return super.uploadLogo(id, file);
  }

  @GetMapping(SolidifyConstants.URL_ID_PLUS_SEP + AouActionName.DOWNLOAD_AVATAR)
  @UserPermissions
  @ResponseBody
  public HttpEntity<StreamingResponseBody> downloadAvatar(@PathVariable String id) throws IOException {
    return super.downloadLogo(id);
  }

  @DeleteMapping(SolidifyConstants.URL_ID_PLUS_SEP + AouActionName.DELETE_AVATAR)
  @CurrentUserPermissions
  @ResponseBody
  public HttpEntity<Person> deleteAvatar(@PathVariable String id) throws IOException {
    return super.deleteLogo(id);
  }

}
