package ch.unige.aou.controller.admin;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.Relation2TiersController;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.JoinResourceContainer;
import ch.unige.solidify.security.UserPermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.publication.PublicationStructure;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.rest.UrlPath;

@UserPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@SuppressWarnings("squid:S4684")
@RequestMapping(UrlPath.ADMIN_PUBLICATIONS + SolidifyConstants.URL_PARENT_ID + ResourceName.STRUCTURES)
public class PublicationStructureController extends Relation2TiersController<Publication, Structure, PublicationStructure> {

  @Override
  @PostMapping(SolidifyConstants.URL_ID)
  @PreAuthorize("@publicationPermissionService.isAllowed(#parentid, 'ADD_DEPOSIT_STRUCTURE')")
  public HttpEntity<JoinResourceContainer<PublicationStructure>> create(@PathVariable String parentid, @PathVariable String id,
          @RequestBody PublicationStructure publicationStructure) {
    return super.create(parentid, id, publicationStructure);
  }

  @Override
  @PostMapping
  @PreAuthorize("@publicationPermissionService.isAllowed(#parentid, 'ADD_DEPOSIT_STRUCTURE')")
  public HttpEntity<List<JoinResourceContainer<PublicationStructure>>> create(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.create(parentid, ids);
  }

  @Override
  @PatchMapping(SolidifyConstants.URL_ID)
  @PreAuthorize("@publicationPermissionService.isAllowed(#parentid, 'UPDATE_DEPOSIT_STRUCTURE')")
  public HttpEntity<JoinResourceContainer<PublicationStructure>> update(@PathVariable String parentid, @PathVariable String id,
          @RequestBody PublicationStructure pStructure) {
    return super.update(parentid, id, pStructure);
  }

  @Override
  @DeleteMapping(SolidifyConstants.URL_ID)
  @PreAuthorize("@publicationPermissionService.isAllowed(#parentid, 'DELETE_DEPOSIT_STRUCTURE')")
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return super.delete(parentid, id);
  }

  @Override
  @DeleteMapping
  @PreAuthorize("@publicationPermissionService.isAllowed(#parentid, 'DELETE_DEPOSIT_STRUCTURE')")
  public HttpStatus deleteList(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.deleteList(parentid, ids);
  }

  @Override
  @PreAuthorize("@publicationPermissionService.isAllowed(#parentid, 'GET')")
  public HttpEntity<JoinResourceContainer<PublicationStructure>> get(@PathVariable String parentid, @PathVariable String id) {
    return super.get(parentid, id);
  }

  @Override
  @UserPermissions
  public HttpEntity<Collection<JoinResourceContainer<PublicationStructure>>> list(@PathVariable String parentid,
          @ModelAttribute Structure filterItem,
          Pageable pageable) {
    return super.list(parentid, filterItem, pageable);
  }
}
