package ch.unige.aou.controller.admin;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

import java.util.Map;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.config.SolidifyProperties;
import ch.unige.solidify.controller.ControllerWithHateoasHome;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.rest.UrlPath;

@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_SYSTEM_PROPERTIES)
public class SystemPropertyController implements ControllerWithHateoasHome {

  private final SystemProperties systemProperties;

  private static class SystemProperties extends RepresentationModel<SystemProperties> {
    private String defaultChecksum;
    private String defaultLicense;
    private String fileSizeLimit;
    private int limitMaxContributors;
    private String orcidClientId;
    private Map<String, String> defaultContributorRoles;

    public SystemProperties(AouProperties aouProperties, SolidifyProperties solidifyProperties) {
      this.defaultChecksum = aouProperties.getParameters().getDefaultChecksum();
      this.defaultLicense = aouProperties.getParameters().getDefaultLicense();
      this.fileSizeLimit = String.valueOf(aouProperties.getParameters().getFileSizeLimit().toBytes());
      this.limitMaxContributors = aouProperties.getMetadata().getImports().getLimitMaxContributors();
      this.orcidClientId = solidifyProperties.getOrcid().getClientId();
      this.defaultContributorRoles = aouProperties.getMetadata().getDefault().getContributorRolesByDepositSubtype();
    }

    public String getDefaultChecksum() {
      return this.defaultChecksum;
    }

    public String getDefaultLicense() {
      return this.defaultLicense;
    }

    public String getFileSizeLimit() {
      return this.fileSizeLimit;
    }

    public int getLimitMaxContributors() {
      return this.limitMaxContributors;
    }

    public String getOrcidClientId() {
      return this.orcidClientId;
    }

    public Map<String, String> getDefaultContributorRoles() {
      return this.defaultContributorRoles;
    }
  }

  public SystemPropertyController(AouProperties aouProperties, SolidifyProperties solidifyProperties) {
    this.systemProperties = new SystemProperties(aouProperties, solidifyProperties);
    this.systemProperties.add(linkTo(methodOn(this.getClass()).home()).withSelfRel());
  }

  @GetMapping
  public HttpEntity<SystemProperties> systemProperties() {
    return new ResponseEntity<>(this.systemProperties, HttpStatus.OK);
  }
}
