package ch.unige.aou.service;

import java.util.NoSuchElementException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import ch.unige.solidify.util.SolidifyTime;

import ch.unige.aou.business.PublicationService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AdminController;
import ch.unige.aou.message.PublicationMessage;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.publication.Publication.PublicationStatus;

@Service
@ConditionalOnBean(AdminController.class)
public class PublicationJmsListener extends MessageProcessor<PublicationMessage> {

  private static final Logger log = LoggerFactory.getLogger(PublicationJmsListener.class);

  private final PublicationProcessingService publicationProcessingService;
  private final PublicationService publicationService;

  public PublicationJmsListener(AouProperties aouProperties, PublicationProcessingService publicationProcessingService,
          PublicationService publicationService) {
    super(aouProperties);
    this.publicationProcessingService = publicationProcessingService;
    this.publicationService = publicationService;
  }

  @Override
  public void processMessage(PublicationMessage publicationMessage) {
    log.info("Reading message {}", publicationMessage);
    try {
      Publication publication = this.publicationService.findOne(publicationMessage.getResId());
      while (this.isPublicationInProgress(publication)) {
        // Process publication
        publication = this.publicationProcessingService.processPublication(publication.getResId());
        SolidifyTime.waitInMilliSeconds(200);
      }
      log.info("Publication '{}' ({}) processed", publication.getResId(), publication.getTitle());

    } catch (NoSuchElementException e) {
      log.error("Cannot find publication ({})", publicationMessage.getResId());
      return;
    }
  }

  private boolean isPublicationInProgress(Publication publication) {
    return (publication.getStatus() == PublicationStatus.SUBMITTED || publication.getStatus() == PublicationStatus.CHECKED
            || publication.getStatus() == PublicationStatus.IN_PREPARATION);
  }

  @JmsListener(destination = "${aou.queue.publications}")
  @Override
  public void receiveMessage(PublicationMessage publicationMessage) {
    this.sendForParallelProcessing(publicationMessage);
  }
}
