package ch.unige.aou.service.admin;

import java.nio.file.Path;
import java.util.List;
import java.util.Map;

import org.springframework.core.io.Resource;

import ch.unige.aou.model.display.StructureWithJoinInfosDTO;
import ch.unige.aou.model.publication.Contributor;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.settings.ResearchGroup;
import ch.unige.aou.service.ResourceClientServiceInterface;

public interface PublicationClientServiceInterface extends ResourceClientServiceInterface<Publication> {
  /**
   * Get list of ResearchGroup
   */
  List<ResearchGroup> getResearchGroups(String publicationId);

  void addResearchGroup(String publicationId, String researchGroupId);

  void removeResearchGroup(String publicationId, String researchGroupId);

  /**
   * Get list of structures
   */
  List<StructureWithJoinInfosDTO> getStructures(String publicationId);

  void addStructure(String publicationId, String structureId);

  void removeStructure(String publicationId, String structureId);

  Publication findByTitle(String title);

  List<Publication> listMyPublications(Map<String, String> properties);

  List<Publication> listValidablePublications(Map<String, String> properties);

  /**
   * Get list of contributors
   */
  List<Contributor> getContributors(String publicationId);

  void submit(String publicationId);

  void submitForApproval(String publicationId);

  void submitForApproval(String publicationId, String validationStructureId);

  void askFeedback(String publicationId, String reason);

  void updateStatusToDelete(String publicationId, String reason);

  void resume(String publicationId);

  void updatePublicationListWithResearchGroup(String researchGroupId, String[] publicationIds);

  void updatePublicationListWithStructure(String structureId, String[] publicationIds);

  /**
   * Imports
   */

  Publication createFromDOI(String doi);

  Publication createFromDOI(String doi, String creatorId);

  Publication createFromPMID(String pmid);

  /**
   * Validation
   */

  String validateMetadata(String publicationId);

  /**
   * Thumbnails
   */

  Publication uploadThumbnail(String publicationId, Resource resource);

  void downloadThumbnail(String publicationId, Path path);

  void deleteThumbnail(String publicationId);

  /**
   * unlink contributor
   */
  void unlinkContributor(String publicationId);

}
