package ch.unige.aou.service.admin;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.util.AbstractRestClientTool;
import ch.unige.solidify.util.CollectionTool;

import ch.unige.aou.model.publication.DocumentFileType;
import ch.unige.aou.model.publication.PublicationSubtype;
import ch.unige.aou.model.publication.PublicationSubtypeDocumentFileTypeDTO;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.ResourceClientService;

@Service
public class PublicationSubtypeClientService extends ResourceClientService<PublicationSubtype>
        implements PublicationSubtypeClientServiceInterface {

  private static final String PUBLICATION_SUBTYPES_PATH = SolidifyConstants.URL_SEP + ResourceName.PUBLICATION_SUBTYPES;

  public PublicationSubtypeClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(PUBLICATION_SUBTYPES_PATH, PublicationSubtype.class, ModuleName.ADMIN, restClientTool, env);
  }

  @Override
  public List<PublicationSubtype> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, PublicationSubtype.class);
  }

  @Override
  public List<PublicationSubtype> findAll() {
    return this.findAll(PublicationSubtype.class);
  }

  @Override
  public PublicationSubtype findByName(String name) {
    final Map<String, String> properties = new HashMap<>();
    properties.put("name", name);
    List<PublicationSubtype> publicationSubtypes = this.searchByProperties(properties);
    if (publicationSubtypes.size() == 1) {
      return publicationSubtypes.get(0);
    }
    throw new SolidifyRuntimeException("Too many publication subtype with name '" + name + "' found");
  }

  @Override
  public List<PublicationSubtypeDocumentFileTypeDTO> findDocumentFileTypes(String resId) {
    return this.findDocumentFileTypes(resId, null);
  }

  @Override
  public List<PublicationSubtypeDocumentFileTypeDTO> findDocumentFileTypes(String resId, DocumentFileType.FileTypeLevel level) {

    String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_ID + SolidifyConstants.URL_SEP
            + AouActionName.LIST_DOCUMENT_FILE_TYPES;
    if (level != null) {
      url += "?level=" + level.name();
    }

    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_ID_FIELD, resId);

    final RestTemplate restTemplate = this.restClientTool.getClient();
    final String jsonString = restTemplate.getForObject(url, String.class, params);
    return CollectionTool.getList(jsonString, PublicationSubtypeDocumentFileTypeDTO.class);
  }
}
