package ch.unige.aou.test.access;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.fail;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.http.HttpStatus;
import org.springframework.web.client.HttpClientErrorException;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.oai.OAISet;
import ch.unige.aou.test.AouTestConstants;

@Disabled("Add these tests when the access module will be in development")
class OAISetIT extends AbstractAccessIT {

  protected void setUser() {
    this.restClientTool.sudoAdmin();
  }

  protected void deleteFixtures() {
    this.oaiSetService.findAll().stream().filter(oaiSet -> oaiSet.getName().startsWith(AouConstants.TEMPORARY_TEST_DATA_LABEL))
            .forEach(oaiSet -> this.oaiSetService.delete(oaiSet.getResId()));
  }

  @Test
  void creationTest() {
    // Create
    final OAISet oaiSet1 = new OAISet();
    oaiSet1.setName(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet1.setSpec(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet1.setQuery("*");
    final OAISet oaiSet2 = this.oaiSetService.create(oaiSet1);

    // Test the creation return
    assertNotNull(oaiSet2, "Cannot create oaiSet");
    assertEquals(oaiSet1.getName(), oaiSet2.getName(), "Problem with oaiSet name");

    // Test creation
    final OAISet oaiSet3 = this.oaiSetService.findOne(oaiSet2.getResId());
    assertNotNull(oaiSet3, "Cannot find oaiSet");
    assertEquals(oaiSet1.getName(), oaiSet3.getName(), "Problem with oaiSet name");
  }

  @Test
  void deleteTest() {
    // Create
    OAISet oaiSet = new OAISet();
    oaiSet.setName(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet.setSpec(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet.setQuery("*");
    final OAISet oaiSet2 = this.oaiSetService.create(oaiSet);
    // Test the creation return
    assertNotNull(oaiSet2, "Cannot create oaiSet");

    // Delete the order
    final String resId = oaiSet2.getResId();
    this.oaiSetService.delete(resId);
    oaiSet = this.oaiSetService.findOne(resId);

    // Test deletion
    assertNull(oaiSet, "Cannot delete oaiSet");
  }

  @Test
  void nameUniquenessTest() {
    final OAISet oaiSet1 = new OAISet();
    oaiSet1.setName("Organizational Units");
    oaiSet1.setSpec(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet1.setQuery("*");
    HttpClientErrorException e = assertThrows(HttpClientErrorException.class, () -> {
      this.oaiSetService.create(oaiSet1);
    });
    assertEquals(HttpStatus.BAD_REQUEST, e.getStatusCode(), "Problem in 'name' uniqueness test");
  }

  @Test
  void specUniquenessTest() {
    final OAISet oaiSet1 = new OAISet();
    oaiSet1.setName(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet1.setSpec(AouConstants.OAI_ORG_UNIT);
    oaiSet1.setQuery("*");
    HttpClientErrorException e = assertThrows(HttpClientErrorException.class, () -> {
      this.oaiSetService.create(oaiSet1);
    });
    assertEquals(HttpStatus.BAD_REQUEST, e.getStatusCode(), "Problem in 'spec' uniqueness test");
  }

  @Test
  void unicityTest() {
    OAISet oaiSet1 = new OAISet();
    oaiSet1.setName(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet1.setSpec(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet1.setQuery("*");
    oaiSet1 = this.oaiSetService.create(oaiSet1);

    OAISet oaiSet2 = new OAISet();
    oaiSet2.setName(oaiSet1.getName());
    oaiSet2.setSpec(oaiSet1.getSpec());
    oaiSet2.setQuery(oaiSet1.getQuery());

    try {
      this.oaiSetService.create(oaiSet2);
      fail("A BAD_REQUEST Http response should be received");
    } catch (final HttpClientErrorException e) {
      assertEquals(HttpStatus.BAD_REQUEST, e.getStatusCode(), "Unicity constraint not satisfied");
    }

  }

  @Test
  void updateTest() {
    // Create
    final OAISet oaiSet = new OAISet();
    oaiSet.setName(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet.setSpec(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    oaiSet.setQuery("*");
    final OAISet oaiSet2 = this.oaiSetService.create(oaiSet);
    // Test the creation return
    assertNotNull(oaiSet2, "Cannot create oaiSet");

    // Update the order
    oaiSet2.setDescription(AouTestConstants.getRandomNameWithTemporaryLabel("oaiSet"));
    final OAISet oaiSet3 = this.oaiSetService.update(oaiSet2.getResId(), oaiSet2);
    assertNotNull(oaiSet3, "Cannot find oaiSet");
    final OAISet oaiSet4 = this.oaiSetService.findOne(oaiSet2.getResId());
    assertNotNull(oaiSet4, "Cannot find oaiSet");

    // Test the update
    assertEquals("Problem with oaiSet name", oaiSet2.getName(), oaiSet4.getName());
    assertEquals("Problem with oaiSet description", oaiSet2.getDescription(), oaiSet4.getDescription());
    this.assertEqualsWithoutNanoSeconds("Problem with oaiSet creation time", oaiSet2.getCreationTime(), oaiSet4.getCreationTime());

  }
}
