package ch.unige.aou;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;

import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;

public class AouConstants {

  private AouConstants() {
    throw new IllegalStateException(SolidifyConstants.TOOL_CLASS);
  }

  // Modules
  public static final String AOU = "AoU";
  public static final String AOU_SOLUTION = "Archive ouverte UNIGE";

  // @formatter:off
  private static final String[] PUBLIC_URLS = {
          "/docs/**",
          "/AoU-logo.png",
          "/**/css/*.css",
          "/**/" + ResourceName.OAI,
          "/**/" + ResourceName.XSL,
          "/**/" + ModuleName.ADMIN + SolidifyConstants.URL_SEP + ResourceName.MODULES,
          "/**/" + ModuleName.ADMIN + SolidifyConstants.URL_SEP + ResourceName.SYSTEM_PROPERTIES,
          "/**/" + ModuleName.ADMIN + SolidifyConstants.URL_SEP + ResourceName.MONITOR + SolidifyConstants.URL_SEP + AouActionName.MONITOR_DATABASE_CONNECTION,
          "/**/" + SolidifyConstants.ORCID + SolidifyConstants.URL_SEP + ActionName.LANDING,
  };
  // @formatter:on

  // @formatter:off
  private static final String[] DISABLED_CSRF_URLS = {
          "/**/" + ResourceName.OAI
  };
  // @formatter:on

  public static String[] getPublicUrls() {
    return PUBLIC_URLS.clone();
  }

  public static String[] getDisabledCsrfUrls() {
    return DISABLED_CSRF_URLS.clone();
  }

  // Default Values
  public static final String ROLE_SUPER_VALIDATOR_ID = "SUPER_VALIDATOR";

  public static final String LANG_CODE_FRENCH = "fre";
  public static final String LANG_CODE_ENGLISH = "eng";
  public static final String LANG_CODE_GERMAN = "ger";

  public static final int ORDER_INCREMENT = 10;

  // Deposit types
  public static final String DEPOSIT_TYPE_ARTICLE_ID = "ARTICLE";
  public static final String DEPOSIT_TYPE_ARTICLE_NAME = "Article";
  public static final String DEPOSIT_TYPE_CONFERENCE_ID = "CONFERENCE";
  public static final String DEPOSIT_TYPE_CONFERENCE_NAME = "Conférence";
  public static final String DEPOSIT_TYPE_DIPLOME_ID = "DIPLOME";
  public static final String DEPOSIT_TYPE_DIPLOME_NAME = "Diplôme";
  public static final String DEPOSIT_TYPE_JOURNAL_ID = "JOURNAL";
  public static final String DEPOSIT_TYPE_JOURNAL_NAME = "Journal";
  public static final String DEPOSIT_TYPE_LIVRE_ID = "LIVRE";
  public static final String DEPOSIT_TYPE_LIVRE_NAME = "Livre";
  public static final String DEPOSIT_TYPE_RAPPORT_ID = "RAPPORT";
  public static final String DEPOSIT_TYPE_RAPPORT_NAME = "Rapport";

  // @formatter:off
  public static final String DEPOSIT_SUBTYPE_ARTICLE_SCIENTIFIQUE_NAME                   = "Article scientifique";
  public static final String DEPOSIT_SUBTYPE_ARTICLE_SCIENTIFIQUE_ID                     = "A1";
  public static final String DEPOSIT_SUBTYPE_ARTICLE_PROFESSIONNEL_NAME                  = "Article professionnel";
  public static final String DEPOSIT_SUBTYPE_ARTICLE_PROFESSIONNEL_ID                    = "A2";
  public static final String DEPOSIT_SUBTYPE_AUTRE_ARTICLE_NAME                          = "Autre article";
  public static final String DEPOSIT_SUBTYPE_AUTRE_ARTICLE_ID                            = "A3";

  public static final String DEPOSIT_SUBTYPE_ACTES_DE_CONFERENCE_NAME                    = "Actes de conférence";
  public static final String DEPOSIT_SUBTYPE_ACTES_DE_CONFERENCE_ID                      = "C1";
  public static final String DEPOSIT_SUBTYPE_PRESENTATION_INTERVENTION_NAME              = "Présentation / Intervention";
  public static final String DEPOSIT_SUBTYPE_PRESENTATION_INTERVENTION_ID                = "C2";
  public static final String DEPOSIT_SUBTYPE_POSTER_NAME                                 = "Poster";
  public static final String DEPOSIT_SUBTYPE_POSTER_ID                                   = "C3";
  public static final String DEPOSIT_SUBTYPE_CHAPITRE_D_ACTES_NAME                       = "Chapitre d'actes";
  public static final String DEPOSIT_SUBTYPE_CHAPITRE_D_ACTES_ID                         = "C4";

  public static final String DEPOSIT_SUBTYPE_THESE_NAME                                  = "Thèse";
  public static final String DEPOSIT_SUBTYPE_THESE_ID                                    = "D1";
  public static final String DEPOSIT_SUBTYPE_MASTER_D_ETUDES_AVANCEES_NAME               = "Master d'études avancées";
  public static final String DEPOSIT_SUBTYPE_MASTER_D_ETUDES_AVANCEES_ID                 = "D2";
  public static final String DEPOSIT_SUBTYPE_MASTER_NAME                                 = "Master";
  public static final String DEPOSIT_SUBTYPE_MASTER_ID                                   = "D3";
  public static final String DEPOSIT_SUBTYPE_THESE_DE_PRIVAT_DOCENT_NAME                 = "Thèse de privat-docent";
  public static final String DEPOSIT_SUBTYPE_THESE_DE_PRIVAT_DOCENT_ID                   = "D4";

  public static final String DEPOSIT_SUBTYPE_NUMERO_DE_REVUE_NAME                        = "Numéro de revue";
  public static final String DEPOSIT_SUBTYPE_NUMERO_DE_REVUE_ID                          = "J1";

  public static final String DEPOSIT_SUBTYPE_LIVRE_NAME                                  = "Livre";
  public static final String DEPOSIT_SUBTYPE_LIVRE_ID                                    = "L1";
  public static final String DEPOSIT_SUBTYPE_OUVRAGE_COLLECTIF_NAME                      = "Ouvrage collectif";
  public static final String DEPOSIT_SUBTYPE_OUVRAGE_COLLECTIF_ID                        = "L2";
  public static final String DEPOSIT_SUBTYPE_CHAPITRE_DE_LIVRE_NAME                      = "Chapitre de livre";
  public static final String DEPOSIT_SUBTYPE_CHAPITRE_DE_LIVRE_ID                        = "L3";
  public static final String DEPOSIT_SUBTYPE_CONTRIBUTION_DICTIONNAIRE_ENCYCLOPEDIE_NAME = "Contribution à un dictionnaire / une encyclopédie";
  public static final String DEPOSIT_SUBTYPE_CONTRIBUTION_DICTIONNAIRE_ENCYCLOPEDIE_ID   = "L5";

  public static final String DEPOSIT_SUBTYPE_RAPPORT_DE_RECHERCHE_NAME                   = "Rapport de recherche";
  public static final String DEPOSIT_SUBTYPE_RAPPORT_DE_RECHERCHE_ID                     = "R1";
  public static final String DEPOSIT_SUBTYPE_RAPPORT_TECHNIQUE_NAME                      = "Rapport technique";
  public static final String DEPOSIT_SUBTYPE_RAPPORT_TECHNIQUE_ID                        = "R2";
  public static final String DEPOSIT_SUBTYPE_WORKING_PAPER_NAME                          = "Working paper";
  public static final String DEPOSIT_SUBTYPE_WORKING_PAPER_ID                            = "R3";
  public static final String DEPOSIT_SUBTYPE_PREPRINT_NAME                               = "Preprint";
  public static final String DEPOSIT_SUBTYPE_PREPRINT_ID                                 = "R4";
  // @formatter:on

  public static final String DEPOSIT_SUB_SUBTYPE_ARTICLE_NAME = "Article";
  public static final String DEPOSIT_SUB_SUBTYPE_COMPTE_RENDU_LIVRE_NAME = "Compte rendu de livre";
  public static final String DEPOSIT_SUB_SUBTYPE_RAPPORT_DU_CAS_NAME = "Rapport de cas";
  public static final String DEPOSIT_SUB_SUBTYPE_COMMENTAIRE_NAME = "Commentaire";
  public static final String DEPOSIT_SUB_SUBTYPE_ARTICLE_DONNEES_NAME = "Article de données";
  public static final String DEPOSIT_SUB_SUBTYPE_EDITORIAL_NAME = "Editorial";
  public static final String DEPOSIT_SUB_SUBTYPE_LETTRE_NAME = "Lettre";
  public static final String DEPOSIT_SUB_SUBTYPE_META_ANALYSIS_NAME = "Méta-analyse";
  public static final String DEPOSIT_SUB_SUBTYPE_REVIEW_LITERATURE_NAME = "Revue de la littérature";
  public static final String DEPOSIT_SUB_SUBTYPE_ARTICLE_VIDEO = "Article vidéo";
  public static final String DEPOSIT_SUB_SUBTYPE_BILLET_DE_BLOG_NAME = "Billet de blog";

  // Database column size
  public static final int DB_SHORT_STRING_LENGTH = 30;
  public static final int DB_MEDIUM_STRING_LENGTH = 100;
  public static final int DB_LONG_STRING_LENGTH = 1024;
  public static final int DB_BIG_STRING_LENGTH = 2048;
  public static final int DB_LARGE_STRING_LENGTH = 4096;
  public static final int DB_DATE_LENGTH = 3;

  // Database column name
  public static final String DB_RES_ID = "resId";
  public static final String DB_INSTITUTION_ID = "institutionId";
  public static final String DB_PERSON_ID = "personId";
  public static final String DB_JOB_ID = "jobId";
  public static final String DB_LANGUAGE_ID = "language_id";
  public static final String DB_LANGUAGE_ENTITY_ID = "language_entity_id";
  public static final String DB_LOGO_ID = "logoId";
  public static final String DB_AVATAR_ID = "avatarId";
  public static final String DB_NAME = "name";
  public static final String DB_PARENT_STRUCTURE_ID = "parent_structure_id";
  public static final String DB_USER_ID = "userId";
  public static final String DB_RESEARCH_GROUP_ID = "researchGroupId";
  public static final String DB_STRUCTURE_ID = "structureId";
  public static final String DB_PUBLICATION_SUBTYPE_ID = "publication_subtype_id";
  public static final String DB_PUBLICATION_SUB_SUBTYPE_ID = "publication_sub_subtype_id";
  public static final String DB_THUMBNAIL_ID = "thumbnail_id";
  public static final String DB_DOCUMENT_FILE_TYPE_ID = "document_file_type_id";
  public static final String DB_CONTRIBUTOR_ROLE_ID = "contributor_role_id";
  public static final String DB_LICENSE_ID = "license_id";
  public static final String DB_LICENSE_GROUP_ID = "license_group_id";
  public static final String DB_PUBLICATION_ID = "publicationId";
  public static final String DB_EVENT_TYPE_ID = "event_type_id";
  public static final String DB_NOTIFICATION_ID = "notification_id";
  public static final String DB_NOTIFICATION_TYPE_ID = "notification_type_id";
  public static final String DB_CN_INDIVIDU = "cnIndividu";
  public static final String DB_ROLE_ID = "role_id";

  // Index
  public static final String REGISTRY = "registry";
  public static final String PRIVATE = "private";

  // Schema
  public static final String XSL_HOME = "xslt";
  public static final String METADATA_V1_TO_V2_XSL = "aou-deposit-metadata-v1-to-v2.xsl";
  public static final String METADATA_V1_TEST_FILE = "deposit_test_v1.xml";

  // Folders
  public static final String METADATA_FOLDER = "metadata";
  public static final String SCHEMA_FOLDER = SolidifyConstants.SCHEMA_HOME;
  public static final String DOC_FOLDER = "documentation";
  public static final String DATA_FOLDER = "data";
  public static final String RESEARCH_DATA_FOLDER = "researchdata";
  public static final String SOFTWARE_FOLDER = "software";
  public static final String PACKAGE = "package";
  public static final String TOMBSTONE = "tombstone";
  public static final String NEW_PACKAGE = "new-package";

  // Metadata field name
  public static final String DATACITE_XML_FIELD = "datacite.xml";
  public static final String DATACITE_FIELD = "datacite";
  public static final String PREMIS_FIELD = "premis";
  public static final String METADATA_VERSION_FIELD = "metadata-version";
  public static final String CREATION_FIELD = "creation";
  public static final String IDENTIFIER_FIELD = "datacite.datacite:resource.datacite:identifier.content";
  public static final String IDENTIFIER_TYPE_FIELD = "datacite.datacite:resource.datacite:identifier.identifierType";
  public static final String RIGHTS_FIELD = "datacite.datacite:resource.datacite:rightsList.datacite:rights";
  public static final String DATES_FIELD = "datacite.datacite:resource.datacite:dates.datacite:date";
  public static final String CREATOR_FIELD = "datacite.datacite:resource.datacite:creators.datacite:creator.datacite:creatorName.content";
  public static final String FORMAT_FIELD = "datacite.datacite:resource.datacite:formats.datacite:format";
  public static final String DATES_AVAILABLE_ATTRIBUTE = "Available";
  public static final String CUSTOM_METADATA_FIELD = "custom-metadata";
  public static final String ITEMS_FIELD = "items";
  public static final String ITEM_FIELD = "item";
  public static final String ID_FIELD = "id";
  public static final String TYPE_FIELD = "type";
  public static final String ARCHIVE_ID_FIELD = "archiveId";
  public static final String CONTENT_FIELD = "content";

  // Metadata values
  public static final String INSTITUTION_UNIGE = "unige";

  public static final String FORMAT_XML = "text/xml";

  // Search
  public static final String SEARCH_QUERY = "query";
  public static final String SEARCH_FIELD = "field";
  public static final String SEARCH_FROM = "from";
  public static final String SEARCH_TO = "to";
  public static final String SEARCH_SET = "set";
  public static final String SEARCH_ORG_UNIT = "orgUnit";
  public static final String SEARCH_CONDITIONS = "conditions";

  // OAI
  public static final String OAI = "oai";
  public static final String OAI_ORG_UNIT = "orgunit";
  public static final String OAI_DOI_REGISTRATION = "doi-registration";
  public static final String OAI_SEP = ":";
  // OAI verbs
  public static final String OAI_IDENTIFY = "Identify";
  public static final String OAI_LIST_METADATA_FORMATS = "ListMetadataFormats";
  public static final String OAI_LIST_IDENTIFIERS = "ListIdentifiers";
  public static final String OAI_GET_RECORD = "GetRecord";
  public static final String OAI_LIST_RECORDS = "ListRecords";
  public static final String OAI_LIST_SETS = "ListSets";
  // OAI parameters
  public static final String OAI_VERB = "verb";
  public static final String OAI_IDENTIFIER = "identifier";
  public static final String OAI_FROM = "from";
  public static final String OAI_TO = "until";
  public static final String OAI_METADATAPREFIX = "metadataPrefix";
  public static final String OAI_SET = "set";
  public static final String OAI_TOKEN = "resumptionToken";
  public static final String OAI_DC = "oai_dc";
  public static final String OAI_DATACITE = "datacite";
  public static final String OAI_SCHEMA_ORG = "schema.org";
  public static final String OAI_VIEW = "smartView";

  // Preservation Job
  public static final String REPORT_ID = "{reportId}";

  // Misc.
  public static final String UNKNOWN = "unknown";

  public static final String REASON = "reason";
  public static final String GRADE = "grade";
  public static final String RATING_TYPE = "ratingType";

  // Tests
  public static final String PERMANENT_TEST_DATA_LABEL = "[Permanent Test Data] ";
  public static final String TEMPORARY_TEST_DATA_LABEL = "[Temporary Test Data] ";
  public static final String NO_REPLY_PREFIX = "noreply-";
  public static final String TEST_RES_ID = "test";
  public static final String TEST_DEPOSIT_JSON_FORM_DATA_TEMPLATE = "{\n"
          + "    \"contributors\": {\n"
          + "        \"contributors\": [\n"
          + "            {\n"
          + "                \"contributor\": {\n"
          + "                    \"cnIndividu\": \"12345\",\n"
          + "                    \"email\": \"john.doe@example.com\",\n"
          + "                    \"firstname\": \"John\",\n"
          + "                    \"institution\": \"unige\",\n"
          + "                    \"lastname\": \"Doe\",\n"
          + "                    \"orcid\": \"1234-1234-1234-1234\",\n"
          + "                    \"role\": \"author\"\n"
          + "                }\n"
          + "            }\n"
          + "        ]\n"
          + "    },\n"
          + "    \"description\": {\n"
          + "        \"languages\": [ {\n"
          + "                \"language\" : \"ENG\"\n"
          + "          }],\n"
          + "        \"dates\": [{\n"
          + "                \"date\": \"24.07.2009\",\n"
          + "                \"type\": \"PUBLICATION\"\n"
          + "          }],\n"
          + "        \"container\": {\n"
          + "            \"identifiers\": {\n"
          + "                \"issn\": \"0028-0836\"\n"
          + "            },\n"
          + "            \"issue\": \"8\",\n"
          + "            \"title\": \"Nature\"\n"
          + "        },\n"
          + "        \"collections\": [{\n"
          + "              \"name\": \"Collection1\"\n"
          + "            }, {\n"
          + "              \"name\": \"Collection2\",\n"
          + "              \"number\": 888\n"
          + "            }"
          + "        ]\n"
          + "    },\n"
          + "    \"type\": {\n"
          + "        \"subsubtype\": \"A1-LETTRE\",\n"
          + "        \"subtype\": \"A1\",\n"
          + "        \"title\": {\"lang\": \"FRA\", \"text\" : \"%s\"},\n"
          + "        \"type\": \"ARTICLE\"\n"
          + "    }\n"
          + "}";

  public static final String STRUCTURE_JSON_V2_0 = "\"academicStructures\": [\n"
          + "      {\n"
          + "        \"code\": \"%s\",\n"
          + "        \"oldCode\": \"%s\",\n"
          + "        \"name\": \"%s\"\n"
          + "      }\n"
          + "    ],";

  public static final String STRUCTURE_JSON_V2_1 = "\"academicStructures\": [\"%s\"],";

  public static final String RESEARCH_GROUP_JSON = "\"groups\": [\n"
          + "      {\n"
          + "        \"code\": \"%s\",\n"
          + "        \"name\": \"%s\"\n"
          + "      }\n"
          + "    ],";
  public static final String TEST_DEPOSIT_TITLE = "Test deposit";
  public static final String TEST_DEPOSIT_JSON_FORM_DATA = String.format(TEST_DEPOSIT_JSON_FORM_DATA_TEMPLATE, TEST_DEPOSIT_TITLE);

  public static final String TEST_DEPOSIT_XML_METADATA_TEMPLATE = "<deposit_doc xmlns=\"https://archive-ouverte.unige.ch/deposit/v2.3\">"
          + "  <type>Diplôme</type>"
          + "  <subtype>Thèse</subtype>"
          + "  <title>%s</title>"
          + "  <languages><language>eng</language></languages>"
          + "  <pages>"
          + "    <paging>p. 12345    -67</paging>"
          + "  </pages>"
          + "</deposit_doc>";

  // Exception error message
  public static final String BAD_SEARCH_REQUEST = "Bad search request";

  // Message processor
  public static final String MESSAGE_PROCESSOR_NAME = "AoU Message Processor";

  // XML
  public static final String XML_NAMESPACE_PREFIX_MAPPER = "com.sun.xml.bind.namespacePrefixMapper";
  public static final String XML_NAMESPACE_DEPOSIT_PREFIX = "aou_deposit";

  // Facets
  public static final String INDEXING_KEYWORD = ".keyword";
  public static final String METADATA_VERSION_FACET = "metadata-versions";
  public static final String ORG_UNIT_FACET = "organizational-units";
  public static final String ACCESS_LEVEL_FACET = "access-levels";
  public static final String DATA_TAG_FACET = "data-tags";
  public static final String CREATOR_FACET = "contributors";
  public static final String FORMAT_FACET = "formats";
  public static final String PATH_FACET = "paths";
  public static final String TYPE_FACET = "types";
  public static final String RETENTION_FACET = "retentions";
  public static final String ARCHIVE_FACET = "archive-units";
  public static final String EXTERNAL_UID = "external-uid";

  // Emails
  public static final String NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX = "[Archive ouverte]";
  public static final String NOTIFICATION_EMAIL_VALIDATION_PREFIX = "[AoU validation]";
}
