package ch.unige.aou.listener;

import javax.persistence.PostPersist;
import javax.persistence.PostRemove;
import javax.persistence.PostUpdate;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.unige.solidify.config.SolidifyEventPublisher;
import ch.unige.solidify.rest.Resource;

import ch.unige.aou.model.StatusHistory;
import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.publication.Publication.PublicationStatus;

public class HistoryListener {
  private static final Logger log = LoggerFactory.getLogger(HistoryListener.class);

  @PostPersist
  private void logCreation(Object object) {
    final Resource<?> res = (Resource<?>) object;

    /*
     * Send a message to store object creation in history
     */
    final StatusHistory stsHistory = new StatusHistory(res.getClass().getSimpleName(), res.getResId(), res.getCreationTime(),
            res.getUpdatedBy());
    log.trace("Sending message {}", stsHistory);
    SolidifyEventPublisher.getPublisher().publishEvent(stsHistory);

    /*
     * Send a message to store first object's status in history
     */
    this.logStatusHistory(object);
  }

  @PostUpdate
  @PostRemove
  private void logStatusHistory(Object object) {
    StatusHistory stsHistory = null;
    if (object instanceof Publication) {
      final Publication publication = (Publication) object;
      if (publication.getStatus() == PublicationStatus.IN_PROGRESS) {
        // Different steps during the creation process
        stsHistory = new StatusHistory(publication.getClass().getSimpleName(), publication.getResId(), publication.getUpdateTime(),
                publication.getStatus().toString(), publication.getStatusMessage(), publication.getFormStep().toString(),
                publication.getUpdatedBy());
      } else {
        stsHistory = new StatusHistory(publication.getClass().getSimpleName(), publication.getResId(), publication.getUpdateTime(),
                publication.getStatus().toString(), publication.getStatusMessage(), publication.getUpdatedBy());
      }

    } else if (object instanceof DocumentFile) {
      final DocumentFile df = (DocumentFile) object;
      String message;
      if (df.getIsImported() && df.getStatus().equals(DocumentFile.DocumentFileStatus.RECEIVED)) {
        if (df.getStatusMessage() == null) {
          message =  "Imported from " + df.getSourceData().toString();
        }
        else {
          message = df.getStatusMessage() + ": Imported from " + df.getSourceData().toString();
        }
      }
      else {
        message = df.getStatusMessage();
      }
      stsHistory = new StatusHistory(df.getClass().getSimpleName(), df.getResId(), df.getUpdateTime(), df.getStatus().toString(),
                message, df.getUpdatedBy());

    }
    if (stsHistory != null) {
      log.trace("Sending event {}", stsHistory);
      SolidifyEventPublisher.getPublisher().publishEvent(stsHistory);
    }
  }

}
