package ch.unige.aou.model;

import java.io.Serializable;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import java.util.Objects;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.util.StringTool;

import ch.unige.aou.AouConstants;


@Entity
public class StatusHistory implements Serializable {
  public static final String CREATED_STATUS = "CREATED";

  private static final long serialVersionUID = 481362136969468794L;

  public static long getSerialVersionUID() {
    return serialVersionUID;
  }

  @NotNull
  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  @Column(length = 3)
  private OffsetDateTime changeTime;

  private String createdBy;

  @Transient
  private String creatorName;

  @Transient
  private String step;

  @Size(max = AouConstants.DB_LONG_STRING_LENGTH)
  private String description;

  @NotNull
  @JsonIgnore
  private String resId;

  @Id
  @GeneratedValue(strategy = GenerationType.AUTO)
  @JsonIgnore
  private long seq;

  @NotNull
  private String status;

  @NotNull
  @JsonIgnore
  private String type;

  public StatusHistory() {
  }

  public StatusHistory(String type, String id, OffsetDateTime datetime, String createdBy) {
    this.type = type;
    this.resId = id;
    this.status = CREATED_STATUS;
    this.changeTime = datetime;
    this.createdBy = createdBy;
  }

  public StatusHistory(String type, String id, OffsetDateTime datetime, String sts, String desc, String createdBy) {
    this.type = type;
    this.resId = id;
    this.status = sts;
    this.changeTime = datetime;
    this.description = desc;
    this.createdBy = createdBy;
  }

  public StatusHistory(String type, String id, OffsetDateTime datetime, String sts, String desc, String step, String createdBy) {
    this.type = type;
    this.resId = id;
    this.status = sts;
    this.changeTime = datetime;
    this.description = desc;
    this.createdBy = createdBy;
    this.step = step;
  }

  public StatusHistory(String type, String id, String sts) {
    this.type = type;
    this.resId = id;
    this.status = sts;
    this.changeTime = OffsetDateTime.now(ZoneOffset.UTC);
    this.createdBy = AouConstants.AOU;
  }

  @Override
  public boolean equals(final Object other) {
    if (!(other instanceof StatusHistory)) {
      return false;
    }
    final StatusHistory castOther = (StatusHistory) other;
    return Objects.equals(this.seq, castOther.seq) && Objects.equals(this.resId, castOther.resId) && Objects.equals(this.type, castOther.type)
            && Objects
            .equals(this.changeTime, castOther.changeTime)
            && Objects.equals(this.status, castOther.status) && Objects
            .equals(this.description, castOther.description);
  }

  public OffsetDateTime getChangeTime() {
    return this.changeTime;
  }

  public String getCreatedBy() {
    return this.createdBy;
  }

  public String getCreatorName() {
    if (StringTool.isNullOrEmpty(this.creatorName)) {
      return this.getCreatedBy();
    } else {
      return this.creatorName;
    }
  }

  public String getDescription() {
    return this.description;
  }

  public String getResId() {
    return this.resId;
  }

  public long getSeq() {
    return this.seq;
  }

  public String getStatus() {
    return this.status;
  }

  public String getType() {
    return this.type;
  }

  public String getStep() {
    return this.step;
  }

  @Override
  public int hashCode() {
    return Objects.hash(this.seq, this.resId, this.type, this.changeTime, this.status, this.description);
  }

  public void setCreatedBy(String createdBy) {
    this.createdBy = createdBy;
  }

  public void setCreatorName(String creatorName) {
    this.creatorName = creatorName;
  }

  public void setDescription(String description) {
    this.description = StringTool.truncateWithEllipsis(description, AouConstants.DB_LONG_STRING_LENGTH);
  }

  @Override
  public String toString() {
    return "StatusHistory [resId=" + this.resId + ", type=" + this.type + ", changeTime=" + this.changeTime + ", status=" + this.status
            + ", description="
            + this.description + "]";
  }

}
