package ch.unige.aou.model.notification;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.validation.constraints.NotNull;

import ch.unige.aou.model.AouResourceNormalized;

@Entity
public class EventType extends AouResourceNormalized<EventType> {

  public static final String PUBLICATION_TO_VALIDATE_ID = "PUBLICATION_TO_VALIDATE";
  public static final EventType PUBLICATION_TO_VALIDATE = new EventType(PUBLICATION_TO_VALIDATE_ID, "Publication to validate");

  public static final String PUBLICATION_CREATED_ID = "PUBLICATION_CREATED";
  public static final EventType PUBLICATION_CREATED = new EventType(PUBLICATION_CREATED_ID, "Publication created");

  public static final String PUBLICATION_SUBMITTED_ID = "PUBLICATION_SUBMITTED";
  public static final EventType PUBLICATION_SUBMITTED = new EventType(PUBLICATION_SUBMITTED_ID, "Publication submitted");

  public static final String PUBLICATION_REJECTED_ID = "PUBLICATION_REJECTED";
  public static final EventType PUBLICATION_REJECTED = new EventType(PUBLICATION_REJECTED_ID, "Publication rejected");

  public static final String COMMENT_IN_PUBLICATION_ID = "COMMENT_IN_PUBLICATION";
  public static final EventType COMMENT_IN_PUBLICATION = new EventType(COMMENT_IN_PUBLICATION_ID, "Comment in publication");

  public static final String PUBLICATION_FEEDBACK_REQUIRED_ID = "PUBLICATION_FEEDBACK_REQUIRED";
  public static final EventType PUBLICATION_FEEDBACK_REQUIRED = new EventType(PUBLICATION_FEEDBACK_REQUIRED_ID, "Publication requires feedback");

  @NotNull
  @Column(unique = true)
  private String name;

  public EventType() {
    //no-arg constructor
  }

  public EventType(String resId, String name) {
    this.setResId(resId);
    this.name = name;
  }

  public String getName() {
    return this.name;
  }

  public void setName(String name) {
    this.setCurrentProperty(name);
  }

  @Override
  public boolean equals(final Object o) {
    if (this == o)
      return true;
    if (!(o instanceof EventType))
      return false;
    final EventType e = (EventType) o;
    return this.getResId().equals(e.getResId()) &&
            Objects.equals(this.name, e.name);
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), this.name);
  }

  public static List<EventType> getAllEventTypes() {
    return Arrays.asList(PUBLICATION_CREATED, PUBLICATION_TO_VALIDATE, PUBLICATION_SUBMITTED, PUBLICATION_REJECTED, COMMENT_IN_PUBLICATION,
            PUBLICATION_FEEDBACK_REQUIRED);
  }

}
