package ch.unige.aou.model.rest;

import java.io.Serializable;
import java.util.Locale;

import com.fasterxml.jackson.databind.JsonNode;

import ch.unige.solidify.util.StringTool;

public class UnigePersonDTO implements Serializable {

  private String cnIndividu;
  private String firstname;
  private String lastname;
  private Integer yearOfBirth;
  private String function;
  private String structure;
  private Boolean isEmployee = false;

  public UnigePersonDTO(String cnIndividu, String firstname, String lastname, Integer yearOfBirth, String function, String structure) {
    this.cnIndividu = cnIndividu;
    this.firstname = firstname;
    this.lastname = lastname;
    this.yearOfBirth = yearOfBirth;
    this.function = function;
    this.structure = structure;
  }

  public UnigePersonDTO(JsonNode node) {
    this.cnIndividu = this.getValue(node, ("entityId"));
    this.firstname = this.getValueFromEditedField(node, "firstname", "firstnameEdited", "usualFirstname");
    this.lastname = this.getValueFromEditedField(node, "lastname", "lastnameEdited", "usualLastname");
    this.function = this.getValue(node, "function");
    this.structure = this.getValue(node, "structure");
    this.isEmployee = this.getCollabValue(node);
    final String birthDate = this.getValue(node, "birth");
    if (birthDate != null) {
      this.yearOfBirth = Integer.parseInt(birthDate.substring(0, 4));
    }
  }

  public String getCnIndividu() {
    return this.cnIndividu;
  }

  public String getFirstname() {
    return this.firstname;
  }

  public String getLastname() {
    return this.lastname;
  }

  public Integer getYearOfBirth() {
    return this.yearOfBirth;
  }

  public String getFunction() {
    return this.function;
  }

  public String getStructure() {
    return this.structure;
  }

  public Boolean isEmployee() {
    return this.isEmployee;
  }

  public void setCnIndividu(String cnIndividu) {
    this.cnIndividu = cnIndividu;
  }

  public void setFirstname(String firstname) {
    this.firstname = firstname;
  }

  public void setLastname(String lastname) {
    this.lastname = lastname;
  }

  public void setYearOfBirth(Integer yearOfBirth) {
    this.yearOfBirth = yearOfBirth;
  }

  public void setFunction(String function) {
    this.function = function;
  }

  public void setStructure(String structure) {
    this.structure = structure;
  }

  public void setIsEmployee(Boolean isEmployee) {
    this.isEmployee = isEmployee;
  }

  private String getValue(JsonNode mainNode, String fieldName) {
    final JsonNode fieldNode = mainNode.get(fieldName);
    if (fieldNode == null) {
      return null;
    }
    return fieldNode.textValue();
  }

  @SuppressWarnings("java:S2447")
  private Boolean getCollabValue(JsonNode node) {
    var indicators = node.get("indicators");
    if (indicators != null) {
      var isCollab = indicators.get("isCollab");
      if (isCollab != null) {
        return isCollab.asBoolean();
      }
    }
    return null;
  }

  private String getValueFromEditedField(JsonNode node, String fieldName, String editedFieldName, String usualFieldName) {
    final String usualValue = this.getValue(node, usualFieldName);
    if (!StringTool.isNullOrEmpty(usualValue)) {
      return usualValue;
    }
    final String editedValue = this.getValue(node, editedFieldName);
    if (!StringTool.isNullOrEmpty(editedValue)) {
      return editedValue;
    }
    String value = this.getValue(node, fieldName);
    if (value == null) {
      return null;
    }
    value = value.trim().replace("\\s+", " ").toLowerCase(Locale.ROOT);
    return this.capitalize(value);
  }

  private String capitalize(String value) {
    StringBuilder result = new StringBuilder();
    boolean separatorFound = false;
    for (int i = 0; i < value.length(); i++) {
      if ((value.charAt(i) == ' ' || value.charAt(i) == '-') && i < value.length() - 1) {
        separatorFound = true;
        result.append(value.charAt(i));
      } else if (i == 0 || separatorFound) {
        separatorFound = false;
        result.append(Character.toUpperCase(value.charAt(i)));
      } else {
        result.append(value.charAt(i));
      }
    }
    return result.toString();
  }

}
