package ch.unige.aou.model.schedule;

import java.time.OffsetDateTime;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityListeners;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.scheduling.support.CronExpression;

import com.fasterxml.jackson.annotation.JsonFormat;

import ch.unige.solidify.util.StringTool;

import ch.unige.aou.listener.HistoryListener;
import ch.unige.aou.model.AouResourceNormalized;

@Entity
@EntityListeners({ HistoryListener.class })
public class ScheduledTask extends AouResourceNormalized<ScheduledTask> {

  @NotNull
  @Size(min = 1)
  @Column(unique = true)
  private String name;

  @NotNull
  @Enumerated(EnumType.STRING)
  private TaskType taskType;

  /**
   * min length is with "* * * * * *" which has 11 chars
   */
  @NotNull
  @Size(min = 11)
  private String cronExpression;

  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  private OffsetDateTime lastExecutionStart;

  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  private OffsetDateTime lastExecutionEnd;

  private Boolean enabled;

  /****************************************************************************************************/

  @Override
  public void init() {
    if (this.enabled == null) {
      this.enabled = true;
    }
  }

  /****************************************************************************************************/

  public String getName() {
    return this.name;
  }

  public void setName(String name) {
    this.setCurrentProperty(name);
  }

  public TaskType getTaskType() {
    return this.taskType;
  }

  public void setTaskType(TaskType taskType) {
    this.setCurrentProperty(taskType);
  }

  public String getCronExpression() {
    return this.cronExpression;
  }

  public void setCronExpression(String cronExpression) {
    this.setCurrentProperty(cronExpression);
  }

  public OffsetDateTime getLastExecutionStart() {
    return this.lastExecutionStart;
  }

  public void setLastExecutionStart(OffsetDateTime lastExecutionStart) {
    this.setCurrentProperty(lastExecutionStart);
  }

  public OffsetDateTime getLastExecutionEnd() {
    return this.lastExecutionEnd;
  }

  public void setLastExecutionEnd(OffsetDateTime lastExecutionEnd) {
    this.setCurrentProperty(lastExecutionEnd);
  }

  public Boolean isEnabled() {
    return this.enabled;
  }

  public void setEnabled(Boolean enabled) {
    this.setCurrentProperty(enabled);
  }

  /****************************************************************************************************/

  public OffsetDateTime getNextExecution() {
    if (StringTool.isNullOrEmpty(this.cronExpression) || !this.enabled) {
      return null;
    }

    CronExpression cronTrigger = CronExpression.parse(this.cronExpression);
    return cronTrigger.next(OffsetDateTime.now());
  }
}
