package ch.unige.aou.model.settings;

import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;

import ch.unige.solidify.SolidifyConstants;

import ch.unige.aou.AouConstants;
import ch.unige.aou.rest.ResourceName;

@Embeddable
@MappedSuperclass
public class Label {

  @NotNull
  @Size(min = 1, max = AouConstants.DB_LONG_STRING_LENGTH)
  @Column(nullable = false)
  private String text;

  @NotNull
  @ManyToOne
  @JoinColumn(name = AouConstants.DB_LANGUAGE_ID, referencedColumnName = SolidifyConstants.DB_RES_ID, nullable = false)
  @JsonIgnore
  private Language language;

  public Label() {
  }

  public Label(String text, Language language) {
    this.text = text;
    this.language = language;
  }

  public String getText() {
    return this.text;
  }

  public Language getLanguage() {
    return this.language;
  }

  @JsonProperty(ResourceName.LANGUAGE)
  public String getLanguageCode() {
    if (this.getLanguage() != null) {
      return this.getLanguage().getCode();
    } else {
      return null;
    }
  }

  /**
   * This setter is required for deserialization
   *
   * @param code
   */
  @JsonProperty(ResourceName.LANGUAGE)
  public void setLanguageCode(String code) {
    Language language = new Language();
    language.setCode(code);
    this.setLanguage(language);
  }

  public void setText(String text) {
    this.text = text;
  }

  public void setLanguage(Language language) {
    this.language = language;
  }
}
