package ch.unige.aou.model.xml.deposit.v1.deserializer;

import java.io.IOException;
import java.math.BigInteger;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v1.Group;
import ch.unige.aou.model.xml.deposit.v1.Groups;

public class GroupsDeserializer extends AouStdDeserializer<Groups> {

  private static final long serialVersionUID = 659865L;

  @Override
  public Groups deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    if (jsonNode instanceof ArrayNode) {
      if (jsonNode.isEmpty()) {
        return null;
      }

      Groups groups = new Groups();
      for (JsonNode groupNode : jsonNode) {
        Group group = new Group();
        JsonNode nameNode = groupNode.get("name");
        JsonNode codeNode = groupNode.get("code");

        //if list contain one element and all properties of this only element are empty we ignore the list
        if (jsonNode.size() == 1 && !this.nodeHasValue(nameNode) && !this.nodeHasValue(codeNode)) {
          return null;
        }
        if (this.nodeHasTextValue(nameNode)) {
          group.setName(nameNode.textValue());
        }
        if (codeNode != null && !codeNode.isNull() && !StringTool.isNullOrEmpty(codeNode.asText())) {
          group.setCode(new BigInteger(codeNode.asText()));
        }
        groups.getGroup().add(group);
      }
      return groups;

    } else {
      throw new SolidifyRuntimeException("groups is not an array");
    }
  }
}
