package ch.unige.aou.model.xml.deposit.v1.deserializer;

import java.io.IOException;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;

import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v1.Right;
import ch.unige.aou.model.xml.deposit.v1.Rights;

public class RightsDeserializer extends AouStdDeserializer<Rights> {

  private static final long serialVersionUID = 659865L;

  @Override
  public Rights deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    if (jsonNode instanceof ArrayNode) {
      if (jsonNode.isEmpty()) {
        return null;
      }

      Rights rights = new Rights();
      for (JsonNode linkNode : jsonNode) {
        Right right = new Right();

        JsonNode uriNode = linkNode.get("uri");
        if (this.nodeHasTextValue(uriNode)) {
          right.setUri(uriNode.textValue());
        }

        rights.getRight().add(right);
      }
      return rights;

    } else {
      throw new SolidifyRuntimeException("rights is not an array");
    }
  }
}
