package ch.unige.aou.test.tool;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

import org.junit.jupiter.api.Test;

import ch.unige.aou.model.tool.CleanTool;

class CleanToolTest {

  @Test
  void cleanTitleTest() {
    assertNull(CleanTool.cleanTitle(null));
    assertEquals("", CleanTool.cleanTitle(""));

    assertEquals("A very interesting title", CleanTool.cleanTitle("A very interesting title"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("A very interesting title."));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A very interesting title   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A    very    interesting    title   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\nA very interesting title\n"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\n\nA very   interesting     title\n\n"));

    assertEquals("A very interesting title", CleanTool.cleanTitle("A very interesting title."));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A very interesting title.   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("    A    very    interesting    title.   "));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\nA very interesting title.\n"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\n\nA very   interesting     title.\n\n"));
    assertEquals("A very interesting title", CleanTool.cleanTitle("\n\nA very \n  interesting  \n   title.\n\n"));
  }

  @Test
  void cleanStringTest() {
    assertNull(CleanTool.cleanString(null));
    assertEquals("", CleanTool.cleanString(""));

    assertEquals("A simple string", CleanTool.cleanString("A simple string"));
    assertEquals("A simple string.", CleanTool.cleanString("A simple string."));
    assertEquals("A simple string", CleanTool.cleanString("   A   simple   string"));
    assertEquals("A simple string.", CleanTool.cleanString("A   simple   string.   "));
    assertEquals("A simple string.", CleanTool.cleanString("\n\nA   simple   string.   \n"));
    assertEquals("A simple string.", CleanTool.cleanString("\n\nA   simple \n\n  string.   \n"));
  }

  @Test
  void cleanPagingTest() {
    assertNull(CleanTool.cleanPaging(null));
    assertEquals("", CleanTool.cleanPaging(""));

    assertEquals("143-145", CleanTool.cleanPaging("143-145"));
    assertEquals("143-145", CleanTool.cleanPaging("      143-145"));
    assertEquals("143-145", CleanTool.cleanPaging("   143-145  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  -  145  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  -  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  -\t45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143 \t -\t 45  "));

    assertEquals("143-143", CleanTool.cleanPaging("143-43"));
    assertEquals("143-143", CleanTool.cleanPaging(" 143  - 43 "));
    assertEquals("143-143", CleanTool.cleanPaging(" 143  -\t 43 "));
    assertEquals("143-143", CleanTool.cleanPaging(" \t143 \t - 43 "));

    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-145"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p.      143-145"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("   p.   143-145"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("   p. \t  143-145\t"));

    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-45"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-   45"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143   -45"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143\t   -45\t"));

    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-5"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143-   5"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143   -5"));
    assertEquals("p. 143-145", CleanTool.cleanPaging("p. 143\t\t-5\t\t\t"));

    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143-   5   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143-   45   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143-   145   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143-   145   test  "));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging(" \t  p.\t143-\t\t\t145\t\t\ttest\t"));

    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -5   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -45   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -145   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143   -145   test   "));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p.\t 143   -145   test   \t"));

    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -  5   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -  45   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("p. 143   -  145   test"));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143   -  145   test  "));
    assertEquals("p. 143-145 test", CleanTool.cleanPaging("   p. 143\t   - \t 145   test  "));

    // other kinds of hyphens
    assertEquals("143-145", CleanTool.cleanPaging("   143  ‒  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  ‑  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  －  45  "));
    assertEquals("143-145", CleanTool.cleanPaging("  \t 143  － \t 45 \t "));
    assertEquals("143-145", CleanTool.cleanPaging("   143  ‒  45  "));

    assertEquals("123456785-123456789", CleanTool.cleanPaging("   123456785  ‒  9  "));
  }

  @Test
  void doNotCleanPagingTest() {
    assertEquals("143-27", CleanTool.cleanPaging("143-27"));
    assertEquals("1243-145", CleanTool.cleanPaging("1243-145"));
    assertEquals("1143-test", CleanTool.cleanPaging("1143-test"));
    assertEquals("bla 1-1 test", CleanTool.cleanPaging("bla 1-1 test"));
    assertEquals("lorem ipsum dolores", CleanTool.cleanPaging("lorem ipsum dolores"));
  }
}
