package ch.unige.aou.service.files;

import java.io.File;
import java.io.IOException;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.encryption.InvalidPasswordException;
import org.apache.pdfbox.text.PDFTextStripper;
import org.springframework.stereotype.Service;

import ch.unige.aou.model.publication.DocumentFile;

/**
 * FileMetadataExtractor implementation that supports only PDF files
 */
@Service
public class PdfBoxMetadataExtractor implements FileMetadataExtractor {

  @Override
  public boolean isValidFile(DocumentFile pdfDocumentFile) {
    try (PDDocument doc = PDDocument.load(new File(pdfDocumentFile.getFinalData()))) {
      return true;
    } catch (InvalidPasswordException invalidPasswordException) {
      return true;
    } catch (IOException e) {
      return false;
    }
  }

  @Override
  public boolean isPasswordProtected(DocumentFile pdfDocumentFile) throws IOException {
    try (PDDocument doc = PDDocument.load(new File(pdfDocumentFile.getFinalData()))) {
      return false;
    } catch (InvalidPasswordException invalidPasswordException) {
      return true;
    }
  }

  @Override
  public String getText(DocumentFile pdfDocumentFile) throws IOException {
    if (!this.isPasswordProtected(pdfDocumentFile)) {
      try (PDDocument doc = PDDocument.load(new File(pdfDocumentFile.getFinalData()))) {
        return new PDFTextStripper().getText(doc);
      }
    } else {
      return null;
    }
  }
}
