package ch.unige.aou.service;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Service;

import ch.unige.aou.model.settings.DownloadToken;
import ch.unige.aou.repository.DownloadTokenRepository;

import ch.unige.solidify.util.HashTool;

@Service
public class DownloadTokenPermissionService {

  private final DownloadTokenRepository downloadTokenRepository;

  public DownloadTokenPermissionService(DownloadTokenRepository downloadTokenRepository) {
    this.downloadTokenRepository = downloadTokenRepository;
  }

  public boolean isAllowed(HttpServletRequest httpServletRequest, String archiveId) {
    String tokenString = null;
    for (Cookie cookie : httpServletRequest.getCookies()) {
      String cookieName = cookie.getName();
      String cookieArchiveId = cookieName.substring(cookieName.indexOf('-') + 1);
      if (cookieArchiveId.equals(archiveId)) {
        tokenString = cookie.getValue();
      }
    }
    if (tokenString == null) {
      return false;
    }
    String tokenHash = HashTool.hash(tokenString);
    DownloadToken downloadToken = this.downloadTokenRepository.findByTokenHash(tokenHash);
    if (downloadToken != null && downloadToken.getArchiveId().equals(archiveId)) {
      return true;
    }
    return false;
  }
}
