package ch.unige.aou.service;

import org.springframework.stereotype.Service;

import ch.unige.solidify.auth.model.ApplicationRole;
import ch.unige.solidify.auth.model.AuthApplicationRole;

import ch.unige.aou.business.PersonService;
import ch.unige.aou.business.UserService;
import ch.unige.aou.model.security.User;

@Service("userPermissionService")
public class UserPermissionService extends AbstractPermissionService {

  private UserService userService;

  public UserPermissionService(PersonService personService, UserService userService) {
    super(personService);
    this.userService = userService;
  }

  public boolean isAllowedToUpdate(String userId, User updatedUser) {
    if (this.hasUserApplicationRole()) {
      return false;
    }
    if (this.hasApplicationRole(AuthApplicationRole.ROOT) || this.hasApplicationRole(AuthApplicationRole.TRUSTED_CLIENT)) {
      return true;
    }
    if (this.hasApplicationRole(AuthApplicationRole.ADMIN)) {
      User beforeUpdateUser = this.userService.findOne(userId);
      ApplicationRole beforeUpdateApplicationRole = beforeUpdateUser.getApplicationRole();
      if (beforeUpdateApplicationRole.equals(updatedUser.getApplicationRole())) {
        return true;
      } else if (AuthApplicationRole.ROOT.equals(beforeUpdateUser.getApplicationRole())) {
        return false;
      } else if (AuthApplicationRole.ROOT.equals(updatedUser.getApplicationRole())) {
        return false;
      } else {
        return true;
      }
    }
    return false;
  }

  public boolean hasValidationRights() {
    String personId = this.personService.getLinkedPersonId(this.getAuthentication());
    return this.personService.hasValidationRights(personId);
  }
}
