package ch.unige.aou.test.resource;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.rest.UrlPath;

public class InstitutionDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_INSTITUTIONS;
  }

  @Override
  public List<String> getResourceLinks() {
    return Arrays.asList(ResourceName.PEOPLE, AouActionName.UPLOAD_LOGO, AouActionName.DOWNLOAD_LOGO);
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the institution")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("description")
        .description("The description of the institution")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("url")
        .description("The URL of the institution website")
        .type(TypeField.URL)
        .customConstraints(CONSTRAINT_VALID_URL)
        .testValue(TEST_URL_VALUE)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("logo")
        .description("Logo of the institution")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("emailSuffixes")
        .description("List of email suffixes")
        .type(TypeField.ARRAY)
        .build());
      // @formatter:on
  }
}
